/*-----------------------------------------------------------------------------
-- gpio.c
--
-- Copyright CERN, 2021
-- This source describes Open Hardware and is licensed under the
-- CERN-OHL-S v2
-- Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
-- designed by INCAA Computers
-----------------------------------------------------------------------------*/

#include "platform.h"

#include "xparameters.h"

#include "xil_types.h"
#include "xil_printf.h"
#include "xil_io.h"

#include "xgpio.h"

#include "gpio.h"

static XGpio gpioInst[XPAR_XGPIO_NUM_INSTANCES];

XStatus gpio_Init(void)
{
	XStatus  Status;
	int inst;

	for (inst=0; inst<XPAR_XGPIO_NUM_INSTANCES; inst++) {
		if (gpioInst[inst].IsReady == 0) {
			Status = XGpio_Initialize(&gpioInst[inst], inst);
			if (Status != XST_SUCCESS) {
				return XST_FAILURE;
			}
		}
	}
	return XST_SUCCESS;
}

uint32_t gpio_ReadBits(int inst, unsigned ch, int pos, uint32_t mask)
{
	uint32_t gpio_ch;
	uint32_t value;

	gpio_ch = XGpio_DiscreteRead(&gpioInst[inst], ch);
	gpio_ch &= (mask);
	value = gpio_ch>>pos;
	return value;
}

uint32_t gpio_ReadBit(int inst, unsigned ch, int pos)
{
	uint32_t mask;
	mask = 1 << pos;
	return gpio_ReadBits(inst, ch, pos, mask);
}

void gpio_WriteBits(int inst, unsigned ch, int pos, uint32_t mask, uint32_t value)
{
	uint32_t gpio_ch;

	gpio_ch = XGpio_DiscreteRead(&gpioInst[inst], ch);
	gpio_ch &= (~mask);
	value <<= pos;
	value &=mask;
	gpio_ch |= value;
	XGpio_DiscreteWrite(&gpioInst[inst], ch, gpio_ch);
}

void gpio_WriteBit(int inst, unsigned ch, int pos, uint32_t value)
{
	gpio_WriteBits(inst, ch, pos, 1 << pos, value);
}


uint32_t gpio_ReadBitsDir(int inst, unsigned ch, int pos, uint32_t mask)
{
	uint32_t gpio_dir;
	uint32_t value;

	gpio_dir = XGpio_GetDataDirection(&gpioInst[inst], ch); // 0=output, 1=input
	gpio_dir &= (mask);
	value = gpio_dir>>pos;
	return value;
}

uint32_t gpio_ReadBitDir(int inst, unsigned ch, int pos)
{
	return gpio_ReadBitsDir(inst, ch, pos, 1 << pos);
}

void gpio_WriteBitsDir(int inst, unsigned ch, int pos, uint32_t mask, uint32_t value)
{
	uint32_t gpio_dir;

	gpio_dir = XGpio_GetDataDirection(&gpioInst[inst], ch); // 0=output, 1=input
	gpio_dir &= (~mask);
	value <<= pos;
	value &=mask;
	gpio_dir |= value;
	XGpio_SetDataDirection(&gpioInst[inst], ch, gpio_dir);
}

void gpio_WriteBitDir(int inst, unsigned ch, int pos, uint32_t value)
{
	gpio_WriteBitsDir(inst, ch, pos, 1 << pos, value);
}

