/*-----------------------------------------------------------------------------
-- gpio.h
--
-- Copyright CERN, 2021
-- This source describes Open Hardware and is licensed under the
-- CERN-OHL-S v2
-- Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
-- designed by INCAA Computers
-----------------------------------------------------------------------------*/

#ifndef SRC_GPIO_H_
#define SRC_GPIO_H_

#include <xstatus.h>

// XPAR_AXI_GPIO_CEX_0_DEVICE_ID channel 0
#define GPIO_CEX_FPGA_GPO_POS		0
#define GPIO_CEX_FPGA_GPO0_POS		0
#define GPIO_CEX_FPGA_GPO1_POS		1
#define GPIO_CEX_FPGA_GPO_SEL_N_POS	2
#define GPIO_TEST1_POS				3
#define GPIO_TEST2_POS				4

#define GPIO_CEX_GPI_POS			GPIO_CEX_FPGA_GPO_POS
#define GPIO_CEX_GPI0_POS			GPIO_CEX_FPGA_GPO0_POS
#define GPIO_CEX_GPI1_POS			GPIO_CEX_FPGA_GPO1_POS

#define GPIO_CEX_FPGA_GPO			0x03
#define GPIO_CEX_FPGA_GPO0			0x01
#define GPIO_CEX_FPGA_GPO1			0x02
#define GPIO_CEX_FPGA_GPO_SEL_N		0x04
#define GPIO_TEST1					0x08
#define GPIO_TEST2					0x10

#define GPIO_CEX_GPI				GPIO_CEX_FPGA_GPO
#define GPIO_CEX_GPI0				GPIO_CEX_FPGA_GPO0
#define GPIO_CEX_GPI1				GPIO_CEX_FPGA_GPO1

// XPAR_AXI_GPIO_CEX_0_DEVICE_ID channel 1
#define GPIO_CEX_FPGA_GPI_POS		0
#define GPIO_CEX_FPGA_GPI0_POS		0
#define GPIO_CEX_FPGA_GPI1_POS		1
#define GPIO_CEX_FPGA_GPI2_POS		2
#define GPIO_CEX_FPGA_GPI3_POS		3

#define GPIO_CEX_GPO_POS			GPIO_CEX_FPGA_GPI_POS
#define GPIO_CEX_GPO0_POS			GPIO_CEX_FPGA_GPI0_POS
#define GPIO_CEX_GPO1_POS			GPIO_CEX_FPGA_GPI1_POS
#define GPIO_CEX_GPO2_POS			GPIO_CEX_FPGA_GPI2_POS
#define GPIO_CEX_GPO3_POS			GPIO_CEX_FPGA_GPI3_POS

#define GPIO_CEX_FPGA_GPI			0x0F
#define GPIO_CEX_FPGA_GPI0			0x01
#define GPIO_CEX_FPGA_GPI1			0x02
#define GPIO_CEX_FPGA_GPI2			0x04
#define GPIO_CEX_FPGA_GPI3			0x08

#define GPIO_CEX_GPO				GPIO_CEX_FPGA_GPI
#define GPIO_CEX_GPO0				GPIO_CEX_FPGA_GPI0
#define GPIO_CEX_GPO1				GPIO_CEX_FPGA_GPI1
#define GPIO_CEX_GPO2				GPIO_CEX_FPGA_GPI2
#define GPIO_CEX_GPO3				GPIO_CEX_FPGA_GPI3

// XPAR_AXI_GPIO_PXI_0_DEVICE_ID channel 0
#define GPIO_PXI_POS				0
#define GPIO_PXI_TRIG_POS			0
#define GPIO_PXI_TRIG0_POS			0
#define GPIO_PXI_TRIG1_POS			1
#define GPIO_PXI_TRIG2_POS			2
#define GPIO_PXI_TRIG3_POS			3
#define GPIO_PXI_TRIG4_POS			4
#define GPIO_PXI_TRIG5_POS			5
#define GPIO_PXI_TRIG6_POS			6
#define GPIO_PXI_TRIG7_POS			7	// PXI_TRIG7 has 4 outputs which, in normal operational situations
#define GPIO_PXI_TRIG7A_POS			8	// should be used to be able to sink/source up to 75mA.
#define GPIO_PXI_TRIG7B_POS			9	// For testing all 4 outputs can be used independently. The gateware
#define GPIO_PXI_TRIG7C_POS			10  // prevents simultaneous driving of the individual outputs with different levels.
#define GPIO_PXI_LBR6_POS			11
#define GPIO_FRONT_TRIG_POS			12
#define GPIO_FRONT_TRIG_TSEL_POS	13

#define GPIO_PXI					0x07FF
#define GPIO_PXI_TRIG				0x00FF
#define GPIO_PXI_TRIG0				0x0001
#define GPIO_PXI_TRIG1				0x0002
#define GPIO_PXI_TRIG2				0x0004
#define GPIO_PXI_TRIG3				0x0008
#define GPIO_PXI_TRIG4				0x0010
#define GPIO_PXI_TRIG5				0x0020
#define GPIO_PXI_TRIG6				0x0040
#define GPIO_PXI_TRIG7				0x0080
#define GPIO_PXI_TRIG7A				0x0100
#define GPIO_PXI_TRIG7B				0x0200
#define GPIO_PXI_TRIG7C				0x0400
#define GPIO_PXI_LBR6				0x0800
#define GPIO_FRONT_TRIG				0x1000
#define GPIO_FRONT_TRIG_TSEL		0x2000

// XPAR_AXI_GPIO_PXI_0_DEVICE_ID channel 1
#define GPIO_PXI_GA_POS				0
#define GPIO_PXI_GA0_POS			0
#define GPIO_PXI_GA1_POS			1
#define GPIO_PXI_GA2_POS			2
#define GPIO_PXI_GA3_POS			3
#define GPIO_PXI_GA4_POS			4
#define GPIO_PXI_STAR_POS			5

#define GPIO_PXI_GA					0x1F
#define GPIO_PXI_GA0				0x01
#define GPIO_PXI_GA1				0x02
#define GPIO_PXI_GA2				0x04
#define GPIO_PXI_GA3				0x08
#define GPIO_PXI_GA4				0x10
#define GPIO_PXI_STAR				0x20

// XPAR_AXI_GPIO_PXI_STAR_0_DEVICE_ID channel 0
#define GPIO_PXI_STAR_REC_POS		0
#define GPIO_PXI_STAR_REC			0xFFFFFFFF

XStatus gpio_Init(void);
uint32_t gpio_ReadBits(int inst, unsigned ch, int pos, uint32_t mask);
uint32_t gpio_ReadBit(int inst, unsigned ch, int pos);
void gpio_WriteBits(int inst, unsigned ch, int pos, uint32_t mask, uint32_t value);
void gpio_WriteBit(int inst, unsigned ch, int pos, uint32_t value);

uint32_t gpio_ReadBitsDir(int inst, unsigned ch, int pos, uint32_t mask);
uint32_t gpio_ReadBitDir(int inst, unsigned ch, int pos);
void gpio_WriteBitsDir(int inst, unsigned ch, int pos, uint32_t mask, uint32_t value);
void gpio_WriteBitDir(int inst, unsigned ch, int pos, uint32_t value);

// FPGA_GPO -> CEX_GPI, FPGA_GPI <- CEX_GPO
static uint32_t inline gpio_GetGpi(void) { return gpio_ReadBits(XPAR_AXI_GPIO_CEX_0_DEVICE_ID, 2, GPIO_CEX_FPGA_GPI_POS, GPIO_CEX_FPGA_GPI); }
static uint32_t inline gpio_GetGpo(void) { return gpio_ReadBits(XPAR_AXI_GPIO_CEX_0_DEVICE_ID, 1, GPIO_CEX_FPGA_GPO_POS, GPIO_CEX_FPGA_GPO); }
static void inline gpio_SetGpo(uint32_t value) { gpio_WriteBits(XPAR_AXI_GPIO_CEX_0_DEVICE_ID, 1, GPIO_CEX_FPGA_GPO_POS, GPIO_CEX_FPGA_GPO, value); }
static uint32_t inline gpio_GetGpoEnable(void) { return gpio_ReadBit(XPAR_AXI_GPIO_CEX_0_DEVICE_ID, 1, GPIO_CEX_FPGA_GPO_SEL_N_POS); }
static void inline gpio_SetGpoEnable(uint32_t value) { gpio_WriteBit(XPAR_AXI_GPIO_CEX_0_DEVICE_ID, 1, GPIO_CEX_FPGA_GPO_SEL_N_POS, value); }

// TEST1 and TEST2
static void inline gpio_SetTest1(uint32_t value) { gpio_WriteBit(XPAR_AXI_GPIO_CEX_0_DEVICE_ID, 1, GPIO_TEST1_POS, value); }
static void inline gpio_SetTest2(uint32_t value) { gpio_WriteBit(XPAR_AXI_GPIO_CEX_0_DEVICE_ID, 1, GPIO_TEST2_POS, value); }

// PXI_TRIG[7:0]
static uint32_t inline gpio_GetPxiDir(void) { return gpio_ReadBitsDir(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_PXI_POS, GPIO_PXI); }
static void inline gpio_SetPxiDir(uint32_t value) { gpio_WriteBitsDir(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_PXI_POS, GPIO_PXI, value); }
static uint32_t inline gpio_GetPxi(void) { return gpio_ReadBits(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_PXI_POS, GPIO_PXI); }
static void inline gpio_SetPxi(uint32_t value) { gpio_WriteBits(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_PXI_POS, GPIO_PXI, value); }

// PXI_LBR6
static uint32_t inline gpio_GetLbrDir(void) { return gpio_ReadBitDir(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_PXI_LBR6_POS); }
static void inline gpio_SetLbrDir(uint32_t value) { gpio_WriteBitDir(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_PXI_LBR6_POS, value); }
static uint32_t inline gpio_GetLbr(void) { return gpio_ReadBit(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_PXI_LBR6_POS); }
static void inline gpio_SetLbr(uint32_t value) { gpio_WriteBit(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_PXI_LBR6_POS, value); }

// Front TRIG
static uint32_t inline gpio_GetTrigDir(void) { return gpio_ReadBitDir(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_FRONT_TRIG_POS); }
static void inline gpio_SetTrigDir(uint32_t value) { gpio_WriteBitDir(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_FRONT_TRIG_POS, value); }
static uint32_t inline gpio_GetTrig(void) { return gpio_ReadBit(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_FRONT_TRIG_POS); }
static void inline gpio_SetTrig(uint32_t value) { gpio_WriteBit(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_FRONT_TRIG_POS, value); }
static uint32_t inline gpio_GetTrigTerm(void) { return gpio_ReadBit(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_FRONT_TRIG_TSEL_POS); }
static void inline gpio_SetTrigTerm(uint32_t value) { gpio_WriteBit(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_FRONT_TRIG_TSEL_POS, value); }
static void inline gpio_EnTrigTerm(void) { gpio_WriteBitDir(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 1, GPIO_FRONT_TRIG_TSEL_POS, 0); }

// PXI_GA
static uint32_t inline gpio_GetPxiGa(void) { return gpio_ReadBits(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 2, GPIO_PXI_GA_POS, GPIO_PXI_GA); }

// PXI_STAR
static uint32_t inline gpio_GetPxiStar(void) { return gpio_ReadBit(XPAR_AXI_GPIO_PXI_0_DEVICE_ID, 2, GPIO_PXI_STAR_POS); }
static uint32_t inline gpio_GetPxiStarRec(void) { return gpio_ReadBits(XPAR_AXI_GPIO_PXI_STAR_0_DEVICE_ID, 1, GPIO_PXI_STAR_REC_POS, GPIO_PXI_STAR_REC); }

#endif /* SRC_GPIO_H_ */
