/*-----------------------------------------------------------------------------
-- tFPGA_app.c
--
-- Copyright CERN, 2021
-- This source describes Open Hardware and is licensed under the
-- CERN-OHL-S v2
-- Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
-- designed by INCAA Computers
-----------------------------------------------------------------------------*/

#include <stdio.h>

#include "xparameters.h"

#include "platform.h"
#include "platform_config.h"

#include "xil_types.h"
#include "xil_cache.h"
#include "xil_printf.h"
#include "xil_exception.h"
#include "xil_io.h"

#include "sys.h"
#include "print.h"
#include "gpio.h"
#include "wd.h"
#include "command.h"

int main()
{
	XStatus Status;
	tick_t ticks, curTicks=0;

	wd_Init();
	init_platform();

	Status = gpio_Init();
	sys_Error(Status);

	print_Splash();

	/* now enable interrupts */
	platform_enable_interrupts();
	wd_Feed();

	cmdInit();

	/* receive and process packets */
	while (1) {

		cmdParser();
#ifndef tFPGA_RELEASE
		ticks = timer_ticks();
		if ((ticks % 5000) == 0) {
			if (ticks != curTicks) {
				xil_printf("*Ticks=%d\n\r", timer_ticks());
				curTicks = ticks;
			}
		}
#endif
		wd_Feed();
	}

	/* never reached */
	cleanup_platform();

	return 0;
}

