
################################################################
# This is a generated script based on design: design_tFPGA
#
# Though there are limitations about the generated script,
# the main purpose of this utility is to make learning
# IP Integrator Tcl commands easier.
################################################################

namespace eval _tcl {
proc get_script_folder {} {
   set script_path [file normalize [info script]]
   set script_folder [file dirname $script_path]
   return $script_folder
}
}
variable script_folder
set script_folder [_tcl::get_script_folder]

################################################################
# Check if script is running in correct Vivado version.
################################################################
set scripts_vivado_version 2018.1
set current_vivado_version [version -short]

if { [string first $scripts_vivado_version $current_vivado_version] == -1 } {
   puts ""
   catch {common::send_msg_id "BD_TCL-109" "ERROR" "This script was generated using Vivado <$scripts_vivado_version> and is being run in <$current_vivado_version> of Vivado. Please run the script in Vivado <$scripts_vivado_version> then open the design in Vivado <$current_vivado_version>. Upgrade the design by running \"Tools => Report => Report IP Status...\", then run write_bd_tcl to create an updated script."}

   return 1
}

################################################################
# START
################################################################

# To test this script, run the following commands from Vivado Tcl console:
# source design_tFPGA_script.tcl


# The design that will be created by this Tcl script contains the following 
# module references:
# t_gpio_interface

# Please add the sources of those modules before sourcing this Tcl script.

# If there is no project opened, this script will create a
# project, but make sure you do not have an existing project
# <./myproj/project_1.xpr> in the current working folder.

set list_projs [get_projects -quiet]
if { $list_projs eq "" } {
   create_project project_1 myproj -part xc7a50tftg256-1
}


# CHANGE DESIGN NAME HERE
variable design_name
set design_name design_tFPGA

# If you do not already have an existing IP Integrator design open,
# you can create a design using the following command:
#    create_bd_design $design_name

# Creating design if needed
set errMsg ""
set nRet 0

set cur_design [current_bd_design -quiet]
set list_cells [get_bd_cells -quiet]

if { ${design_name} eq "" } {
   # USE CASES:
   #    1) Design_name not set

   set errMsg "Please set the variable <design_name> to a non-empty value."
   set nRet 1

} elseif { ${cur_design} ne "" && ${list_cells} eq "" } {
   # USE CASES:
   #    2): Current design opened AND is empty AND names same.
   #    3): Current design opened AND is empty AND names diff; design_name NOT in project.
   #    4): Current design opened AND is empty AND names diff; design_name exists in project.

   if { $cur_design ne $design_name } {
      common::send_msg_id "BD_TCL-001" "INFO" "Changing value of <design_name> from <$design_name> to <$cur_design> since current design is empty."
      set design_name [get_property NAME $cur_design]
   }
   common::send_msg_id "BD_TCL-002" "INFO" "Constructing design in IPI design <$cur_design>..."

} elseif { ${cur_design} ne "" && $list_cells ne "" && $cur_design eq $design_name } {
   # USE CASES:
   #    5) Current design opened AND has components AND same names.

   set errMsg "Design <$design_name> already exists in your project, please set the variable <design_name> to another value."
   set nRet 1
} elseif { [get_files -quiet ${design_name}.bd] ne "" } {
   # USE CASES: 
   #    6) Current opened design, has components, but diff names, design_name exists in project.
   #    7) No opened design, design_name exists in project.

   set errMsg "Design <$design_name> already exists in your project, please set the variable <design_name> to another value."
   set nRet 2

} else {
   # USE CASES:
   #    8) No opened design, design_name not in project.
   #    9) Current opened design, has components, but diff names, design_name not in project.

   common::send_msg_id "BD_TCL-003" "INFO" "Currently there is no design <$design_name> in project, so creating one..."

   create_bd_design $design_name

   common::send_msg_id "BD_TCL-004" "INFO" "Making design <$design_name> as current_bd_design."
   current_bd_design $design_name

}

common::send_msg_id "BD_TCL-005" "INFO" "Currently the variable <design_name> is equal to \"$design_name\"."

if { $nRet != 0 } {
   catch {common::send_msg_id "BD_TCL-114" "ERROR" $errMsg}
   return $nRet
}

##################################################################
# DESIGN PROCs
##################################################################



# Procedure to create entire design; Provide argument to make
# procedure reusable. If parentCell is "", will use root.
proc create_root_design { parentCell } {

  variable script_folder
  variable design_name

  if { $parentCell eq "" } {
     set parentCell [get_bd_cells /]
  }

  # Get object for parentCell
  set parentObj [get_bd_cells $parentCell]
  if { $parentObj == "" } {
     catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
     return
  }

  # Make sure parentObj is hier blk
  set parentType [get_property TYPE $parentObj]
  if { $parentType ne "hier" } {
     catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
     return
  }

  # Save current instance; Restore later
  set oldCurInst [current_bd_instance .]

  # Set parent object as current
  current_bd_instance $parentObj


  # Create interface ports

  # Create ports
  set RESET_N [ create_bd_port -dir I -type rst RESET_N ]
  set fpga_gpi [ create_bd_port -dir I -from 3 -to 0 fpga_gpi ]
  set fpga_gpo [ create_bd_port -dir O -from 1 -to 0 fpga_gpo ]
  set fpga_gpo_sel_N [ create_bd_port -dir O fpga_gpo_sel_N ]
  set pxi_clk10 [ create_bd_port -dir I -type clk pxi_clk10 ]
  set_property -dict [ list \
   CONFIG.FREQ_HZ {10000000} \
 ] $pxi_clk10
  set pxi_ga [ create_bd_port -dir I -from 4 -to 0 pxi_ga ]
  set pxi_lbr6 [ create_bd_port -dir IO pxi_lbr6 ]
  set pxi_star [ create_bd_port -dir I pxi_star ]
  set pxi_trig [ create_bd_port -dir IO -from 7 -to 0 pxi_trig ]
  set pxi_trig7aux [ create_bd_port -dir IO -from 2 -to 0 pxi_trig7aux ]
  set ser_rx [ create_bd_port -dir O ser_rx ]
  set ser_tx [ create_bd_port -dir I ser_tx ]
  set test [ create_bd_port -dir O -from 2 -to 1 test ]
  set trig_in [ create_bd_port -dir I trig_in ]
  set trig_oe [ create_bd_port -dir O trig_oe ]
  set trig_out [ create_bd_port -dir O trig_out ]
  set trig_tsel [ create_bd_port -dir O trig_tsel ]

  # Create instance: axi_bram_ctrl_0, and set properties
  set axi_bram_ctrl_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_bram_ctrl:4.0 axi_bram_ctrl_0 ]

  # Create instance: axi_gpio_cex_0, and set properties
  set axi_gpio_cex_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_gpio:2.0 axi_gpio_cex_0 ]
  set_property -dict [ list \
   CONFIG.C_ALL_INPUTS {0} \
   CONFIG.C_ALL_INPUTS_2 {1} \
   CONFIG.C_ALL_OUTPUTS {1} \
   CONFIG.C_ALL_OUTPUTS_2 {0} \
   CONFIG.C_GPIO2_WIDTH {4} \
   CONFIG.C_GPIO_WIDTH {5} \
   CONFIG.C_IS_DUAL {1} \
 ] $axi_gpio_cex_0

  # Create instance: axi_gpio_pxi_0, and set properties
  set axi_gpio_pxi_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_gpio:2.0 axi_gpio_pxi_0 ]
  set_property -dict [ list \
   CONFIG.C_ALL_INPUTS_2 {1} \
   CONFIG.C_GPIO2_WIDTH {6} \
   CONFIG.C_GPIO_WIDTH {14} \
   CONFIG.C_IS_DUAL {1} \
 ] $axi_gpio_pxi_0

  # Create instance: axi_gpio_pxi_star_0, and set properties
  set axi_gpio_pxi_star_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_gpio:2.0 axi_gpio_pxi_star_0 ]
  set_property -dict [ list \
   CONFIG.C_ALL_INPUTS {1} \
   CONFIG.C_ALL_INPUTS_2 {1} \
   CONFIG.C_ALL_OUTPUTS {0} \
   CONFIG.C_ALL_OUTPUTS_2 {0} \
   CONFIG.C_GPIO2_WIDTH {32} \
   CONFIG.C_GPIO_WIDTH {32} \
   CONFIG.C_IS_DUAL {0} \
 ] $axi_gpio_pxi_star_0

  # Create instance: axi_intc_0, and set properties
  set axi_intc_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_intc:4.1 axi_intc_0 ]
  set_property -dict [ list \
   CONFIG.C_HAS_FAST {1} \
   CONFIG.C_KIND_OF_INTR {0xFFFFFE08} \
 ] $axi_intc_0

  # Create instance: axi_mem_intercon, and set properties
  set axi_mem_intercon [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_interconnect:2.1 axi_mem_intercon ]
  set_property -dict [ list \
   CONFIG.NUM_MI {1} \
   CONFIG.NUM_SI {3} \
 ] $axi_mem_intercon

  # Create instance: axi_periph_intercon, and set properties
  set axi_periph_intercon [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_interconnect:2.1 axi_periph_intercon ]
  set_property -dict [ list \
   CONFIG.NUM_MI {8} \
 ] $axi_periph_intercon

  # Create instance: axi_timebase_wdt_0, and set properties
  set axi_timebase_wdt_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_timebase_wdt:3.0 axi_timebase_wdt_0 ]
  set_property -dict [ list \
   CONFIG.C_WDT_INTERVAL {29} \
 ] $axi_timebase_wdt_0

  # Create instance: axi_timer_0, and set properties
  set axi_timer_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_timer:2.0 axi_timer_0 ]

  # Create instance: axi_uartlite_0, and set properties
  set axi_uartlite_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_uartlite:2.0 axi_uartlite_0 ]
  set_property -dict [ list \
   CONFIG.C_BAUDRATE {115200} \
 ] $axi_uartlite_0

  # Create instance: blk_mem_gen_0, and set properties
  set blk_mem_gen_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:blk_mem_gen:8.4 blk_mem_gen_0 ]
  set_property -dict [ list \
   CONFIG.Enable_B {Use_ENB_Pin} \
   CONFIG.Memory_Type {True_Dual_Port_RAM} \
   CONFIG.Port_B_Clock {100} \
   CONFIG.Port_B_Enable_Rate {100} \
   CONFIG.Port_B_Write_Rate {50} \
   CONFIG.Use_RSTB_Pin {true} \
 ] $blk_mem_gen_0

  # Create instance: clk_wiz_0, and set properties
  set clk_wiz_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:clk_wiz:6.0 clk_wiz_0 ]
  set_property -dict [ list \
   CONFIG.CLKIN1_JITTER_PS {100.000} \
   CONFIG.CLKIN1_UI_JITTER {100.000} \
   CONFIG.CLKIN2_JITTER_PS {100.000} \
   CONFIG.CLKIN2_UI_JITTER {100.000} \
   CONFIG.CLKOUT1_DRIVES {BUFG} \
   CONFIG.CLKOUT1_JITTER {597.520} \
   CONFIG.CLKOUT1_PHASE_ERROR {892.144} \
   CONFIG.CLKOUT2_DRIVES {BUFG} \
   CONFIG.CLKOUT2_JITTER {165.419} \
   CONFIG.CLKOUT2_PHASE_ERROR {96.948} \
   CONFIG.CLKOUT2_REQUESTED_OUT_FREQ {100.000} \
   CONFIG.CLKOUT2_USED {false} \
   CONFIG.CLKOUT3_DRIVES {BUFG} \
   CONFIG.CLKOUT3_JITTER {124.615} \
   CONFIG.CLKOUT3_PHASE_ERROR {96.948} \
   CONFIG.CLKOUT3_REQUESTED_OUT_FREQ {100.000} \
   CONFIG.CLKOUT3_USED {false} \
   CONFIG.CLKOUT4_DRIVES {BUFG} \
   CONFIG.CLKOUT4_JITTER {124.615} \
   CONFIG.CLKOUT4_PHASE_ERROR {96.948} \
   CONFIG.CLKOUT4_REQUESTED_OUT_FREQ {100.000} \
   CONFIG.CLKOUT4_USED {false} \
   CONFIG.CLKOUT5_DRIVES {BUFG} \
   CONFIG.CLKOUT6_DRIVES {BUFG} \
   CONFIG.CLKOUT7_DRIVES {BUFG} \
   CONFIG.FEEDBACK_SOURCE {FDBK_AUTO} \
   CONFIG.JITTER_OPTIONS {PS} \
   CONFIG.MMCM_CLKFBOUT_MULT_F {63.750} \
   CONFIG.MMCM_CLKIN1_PERIOD {100.000} \
   CONFIG.MMCM_CLKIN2_PERIOD {10.0} \
   CONFIG.MMCM_CLKOUT0_DIVIDE_F {6.375} \
   CONFIG.MMCM_CLKOUT1_DIVIDE {1} \
   CONFIG.MMCM_CLKOUT2_DIVIDE {1} \
   CONFIG.MMCM_CLKOUT3_DIVIDE {1} \
   CONFIG.MMCM_COMPENSATION {ZHOLD} \
   CONFIG.MMCM_DIVCLK_DIVIDE {1} \
   CONFIG.MMCM_REF_JITTER1 {0.001} \
   CONFIG.MMCM_REF_JITTER2 {0.010} \
   CONFIG.NUM_OUT_CLKS {1} \
   CONFIG.PRIMITIVE {MMCM} \
   CONFIG.PRIM_IN_FREQ {10.000} \
   CONFIG.PRIM_SOURCE {Single_ended_clock_capable_pin} \
   CONFIG.RESET_PORT {resetn} \
   CONFIG.RESET_TYPE {ACTIVE_LOW} \
   CONFIG.USE_LOCKED {true} \
 ] $clk_wiz_0

  # Create instance: mdm_0, and set properties
  set mdm_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:mdm:3.2 mdm_0 ]

  # Create instance: microblaze_0, and set properties
  set microblaze_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:microblaze:10.0 microblaze_0 ]
  set_property -dict [ list \
   CONFIG.C_ADDR_TAG_BITS {5} \
   CONFIG.C_CACHE_BYTE_SIZE {4096} \
   CONFIG.C_DCACHE_ADDR_TAG {5} \
   CONFIG.C_DCACHE_BASEADDR {0x0000000000000000} \
   CONFIG.C_DCACHE_BYTE_SIZE {4096} \
   CONFIG.C_DCACHE_HIGHADDR {0x000000000001FFFF} \
   CONFIG.C_DCACHE_LINE_LEN {8} \
   CONFIG.C_DCACHE_USE_WRITEBACK {1} \
   CONFIG.C_DCACHE_VICTIMS {4} \
   CONFIG.C_D_AXI {1} \
   CONFIG.C_D_LMB {0} \
   CONFIG.C_ICACHE_BASEADDR {0x0000000000000000} \
   CONFIG.C_ICACHE_HIGHADDR {0x000000000001FFFF} \
   CONFIG.C_ICACHE_LINE_LEN {8} \
   CONFIG.C_ICACHE_STREAMS {1} \
   CONFIG.C_ICACHE_VICTIMS {4} \
   CONFIG.C_I_LMB {0} \
   CONFIG.C_MMU_ZONES {2} \
   CONFIG.C_USE_BARREL {1} \
   CONFIG.C_USE_BRANCH_TARGET_CACHE {1} \
   CONFIG.C_USE_DCACHE {1} \
   CONFIG.C_USE_DIV {1} \
   CONFIG.C_USE_FPU {2} \
   CONFIG.C_USE_HW_MUL {2} \
   CONFIG.C_USE_ICACHE {1} \
   CONFIG.C_USE_MSR_INSTR {1} \
   CONFIG.C_USE_PCMP_INSTR {1} \
   CONFIG.G_TEMPLATE_LIST {2} \
 ] $microblaze_0

  # Create instance: proc_sys_reset_0, and set properties
  set proc_sys_reset_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:proc_sys_reset:5.0 proc_sys_reset_0 ]
  set_property -dict [ list \
   CONFIG.C_AUX_RESET_HIGH {1} \
 ] $proc_sys_reset_0

  # Create instance: t_gpio_interface_0, and set properties
  set block_name t_gpio_interface
  set block_cell_name t_gpio_interface_0
  if { [catch {set t_gpio_interface_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $t_gpio_interface_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
  
  # Create instance: xlconcat_0, and set properties
  set xlconcat_0 [ create_bd_cell -type ip -vlnv xilinx.com:ip:xlconcat:2.1 xlconcat_0 ]
  set_property -dict [ list \
   CONFIG.NUM_PORTS {3} \
 ] $xlconcat_0

  # Create interface connections
  connect_bd_intf_net -intf_net S01_AXI_1 [get_bd_intf_pins axi_mem_intercon/S01_AXI] [get_bd_intf_pins microblaze_0/M_AXI_IC]
  connect_bd_intf_net -intf_net axi_bram_ctrl_0_BRAM_PORTA [get_bd_intf_pins axi_bram_ctrl_0/BRAM_PORTA] [get_bd_intf_pins blk_mem_gen_0/BRAM_PORTA]
  connect_bd_intf_net -intf_net axi_bram_ctrl_0_BRAM_PORTB [get_bd_intf_pins axi_bram_ctrl_0/BRAM_PORTB] [get_bd_intf_pins blk_mem_gen_0/BRAM_PORTB]
  connect_bd_intf_net -intf_net axi_intc_0_interrupt [get_bd_intf_pins axi_intc_0/interrupt] [get_bd_intf_pins microblaze_0/INTERRUPT]
  connect_bd_intf_net -intf_net axi_mem_interconnect_0_M00_AXI [get_bd_intf_pins axi_bram_ctrl_0/S_AXI] [get_bd_intf_pins axi_mem_intercon/M00_AXI]
  connect_bd_intf_net -intf_net axi_periph_intercon_M01_AXI [get_bd_intf_pins axi_periph_intercon/M01_AXI] [get_bd_intf_pins axi_timer_0/S_AXI]
  connect_bd_intf_net -intf_net axi_periph_intercon_M02_AXI [get_bd_intf_pins axi_periph_intercon/M02_AXI] [get_bd_intf_pins axi_timebase_wdt_0/S_AXI]
  connect_bd_intf_net -intf_net axi_periph_intercon_M03_AXI [get_bd_intf_pins axi_periph_intercon/M03_AXI] [get_bd_intf_pins axi_uartlite_0/S_AXI]
  connect_bd_intf_net -intf_net axi_periph_intercon_M04_AXI [get_bd_intf_pins axi_gpio_cex_0/S_AXI] [get_bd_intf_pins axi_periph_intercon/M04_AXI]
  connect_bd_intf_net -intf_net axi_periph_intercon_M05_AXI [get_bd_intf_pins axi_gpio_pxi_0/S_AXI] [get_bd_intf_pins axi_periph_intercon/M05_AXI]
  connect_bd_intf_net -intf_net axi_periph_intercon_M06_AXI [get_bd_intf_pins axi_gpio_pxi_star_0/S_AXI] [get_bd_intf_pins axi_periph_intercon/M06_AXI]
  connect_bd_intf_net -intf_net axi_periph_intercon_M07_AXI [get_bd_intf_pins axi_mem_intercon/S02_AXI] [get_bd_intf_pins axi_periph_intercon/M07_AXI]
  connect_bd_intf_net -intf_net axi_periph_interconnect_0_M00_AXI [get_bd_intf_pins axi_intc_0/s_axi] [get_bd_intf_pins axi_periph_intercon/M00_AXI]
  connect_bd_intf_net -intf_net mdm_0_MBDEBUG_0 [get_bd_intf_pins mdm_0/MBDEBUG_0] [get_bd_intf_pins microblaze_0/DEBUG]
  connect_bd_intf_net -intf_net microblaze_0_M_AXI_DC [get_bd_intf_pins axi_mem_intercon/S00_AXI] [get_bd_intf_pins microblaze_0/M_AXI_DC]
  connect_bd_intf_net -intf_net microblaze_0_M_AXI_DP [get_bd_intf_pins axi_periph_intercon/S00_AXI] [get_bd_intf_pins microblaze_0/M_AXI_DP]

  # Create port connections
  connect_bd_net -net Net [get_bd_ports pxi_trig] [get_bd_pins t_gpio_interface_0/pxi_trig]
  connect_bd_net -net Net1 [get_bd_ports pxi_lbr6] [get_bd_pins t_gpio_interface_0/pxi_lbr6]
  connect_bd_net -net Net2 [get_bd_ports pxi_trig7aux] [get_bd_pins t_gpio_interface_0/pxi_trig7aux]
  connect_bd_net -net axi_gpio_cex_0_gpio_io_o [get_bd_pins axi_gpio_cex_0/gpio_io_o] [get_bd_pins t_gpio_interface_0/gpio_cex]
  connect_bd_net -net axi_gpio_pxi_0_gpio_io_o [get_bd_pins axi_gpio_pxi_0/gpio_io_o] [get_bd_pins t_gpio_interface_0/gpio_o_pxi]
  connect_bd_net -net axi_gpio_pxi_0_gpio_io_t [get_bd_pins axi_gpio_pxi_0/gpio_io_t] [get_bd_pins t_gpio_interface_0/gpio_t_pxi]
  connect_bd_net -net axi_timebase_wdt_0_wdt_interrupt [get_bd_pins axi_timebase_wdt_0/wdt_interrupt] [get_bd_pins xlconcat_0/In2]
  connect_bd_net -net axi_timebase_wdt_0_wdt_reset [get_bd_pins axi_timebase_wdt_0/wdt_reset] [get_bd_pins proc_sys_reset_0/aux_reset_in]
  connect_bd_net -net axi_timer_0_interrupt [get_bd_pins axi_timer_0/interrupt] [get_bd_pins xlconcat_0/In1]
  connect_bd_net -net axi_uartlite_0_interrupt [get_bd_pins axi_uartlite_0/interrupt] [get_bd_pins xlconcat_0/In0]
  connect_bd_net -net axi_uartlite_0_tx [get_bd_ports ser_rx] [get_bd_pins axi_uartlite_0/tx]
  connect_bd_net -net clk_in1_0_1 [get_bd_ports pxi_clk10] [get_bd_pins clk_wiz_0/clk_in1]
  connect_bd_net -net clk_wiz_0_clk_out1 [get_bd_pins axi_bram_ctrl_0/s_axi_aclk] [get_bd_pins axi_gpio_cex_0/s_axi_aclk] [get_bd_pins axi_gpio_pxi_0/s_axi_aclk] [get_bd_pins axi_gpio_pxi_star_0/s_axi_aclk] [get_bd_pins axi_intc_0/processor_clk] [get_bd_pins axi_intc_0/s_axi_aclk] [get_bd_pins axi_mem_intercon/ACLK] [get_bd_pins axi_mem_intercon/M00_ACLK] [get_bd_pins axi_mem_intercon/S00_ACLK] [get_bd_pins axi_mem_intercon/S01_ACLK] [get_bd_pins axi_mem_intercon/S02_ACLK] [get_bd_pins axi_periph_intercon/ACLK] [get_bd_pins axi_periph_intercon/M00_ACLK] [get_bd_pins axi_periph_intercon/M01_ACLK] [get_bd_pins axi_periph_intercon/M02_ACLK] [get_bd_pins axi_periph_intercon/M03_ACLK] [get_bd_pins axi_periph_intercon/M04_ACLK] [get_bd_pins axi_periph_intercon/M05_ACLK] [get_bd_pins axi_periph_intercon/M06_ACLK] [get_bd_pins axi_periph_intercon/M07_ACLK] [get_bd_pins axi_periph_intercon/S00_ACLK] [get_bd_pins axi_timebase_wdt_0/s_axi_aclk] [get_bd_pins axi_timer_0/s_axi_aclk] [get_bd_pins axi_uartlite_0/s_axi_aclk] [get_bd_pins clk_wiz_0/clk_out1] [get_bd_pins microblaze_0/Clk] [get_bd_pins proc_sys_reset_0/slowest_sync_clk] [get_bd_pins t_gpio_interface_0/s_axi_aclk]
  connect_bd_net -net clk_wiz_0_locked [get_bd_pins clk_wiz_0/locked] [get_bd_pins proc_sys_reset_0/dcm_locked]
  connect_bd_net -net fpga_gpi_0_1 [get_bd_ports fpga_gpi] [get_bd_pins t_gpio_interface_0/fpga_gpi]
  connect_bd_net -net mdm_0_Debug_SYS_Rst [get_bd_pins mdm_0/Debug_SYS_Rst] [get_bd_pins proc_sys_reset_0/mb_debug_sys_rst]
  connect_bd_net -net proc_sys_reset_0_interconnect_aresetn [get_bd_pins axi_mem_intercon/ARESETN] [get_bd_pins axi_periph_intercon/ARESETN] [get_bd_pins proc_sys_reset_0/interconnect_aresetn]
  connect_bd_net -net proc_sys_reset_0_mb_reset [get_bd_pins axi_intc_0/processor_rst] [get_bd_pins microblaze_0/Reset] [get_bd_pins proc_sys_reset_0/mb_reset]
  connect_bd_net -net proc_sys_reset_0_peripheral_aresetn [get_bd_pins axi_bram_ctrl_0/s_axi_aresetn] [get_bd_pins axi_gpio_cex_0/s_axi_aresetn] [get_bd_pins axi_gpio_pxi_0/s_axi_aresetn] [get_bd_pins axi_gpio_pxi_star_0/s_axi_aresetn] [get_bd_pins axi_intc_0/s_axi_aresetn] [get_bd_pins axi_mem_intercon/M00_ARESETN] [get_bd_pins axi_mem_intercon/S00_ARESETN] [get_bd_pins axi_mem_intercon/S01_ARESETN] [get_bd_pins axi_mem_intercon/S02_ARESETN] [get_bd_pins axi_periph_intercon/M00_ARESETN] [get_bd_pins axi_periph_intercon/M01_ARESETN] [get_bd_pins axi_periph_intercon/M02_ARESETN] [get_bd_pins axi_periph_intercon/M03_ARESETN] [get_bd_pins axi_periph_intercon/M04_ARESETN] [get_bd_pins axi_periph_intercon/M05_ARESETN] [get_bd_pins axi_periph_intercon/M06_ARESETN] [get_bd_pins axi_periph_intercon/M07_ARESETN] [get_bd_pins axi_periph_intercon/S00_ARESETN] [get_bd_pins axi_timebase_wdt_0/s_axi_aresetn] [get_bd_pins axi_timer_0/s_axi_aresetn] [get_bd_pins axi_uartlite_0/s_axi_aresetn] [get_bd_pins proc_sys_reset_0/peripheral_aresetn] [get_bd_pins t_gpio_interface_0/s_axi_aresetn]
  connect_bd_net -net pxi_ga_0_1 [get_bd_ports pxi_ga] [get_bd_pins t_gpio_interface_0/pxi_ga]
  connect_bd_net -net pxi_star_0_1 [get_bd_ports pxi_star] [get_bd_pins t_gpio_interface_0/pxi_star]
  connect_bd_net -net resetn_0_1 [get_bd_ports RESET_N] [get_bd_pins clk_wiz_0/resetn] [get_bd_pins proc_sys_reset_0/ext_reset_in]
  connect_bd_net -net rx_0_1 [get_bd_ports ser_tx] [get_bd_pins axi_uartlite_0/rx]
  connect_bd_net -net t_gpio_interface_0_fpga_gpo [get_bd_ports fpga_gpo] [get_bd_pins t_gpio_interface_0/fpga_gpo]
  connect_bd_net -net t_gpio_interface_0_fpga_gpo_sel_N [get_bd_ports fpga_gpo_sel_N] [get_bd_pins t_gpio_interface_0/fpga_gpo_sel_N]
  connect_bd_net -net t_gpio_interface_0_gpio2_cex [get_bd_pins axi_gpio_cex_0/gpio2_io_i] [get_bd_pins t_gpio_interface_0/gpio2_cex]
  connect_bd_net -net t_gpio_interface_0_gpio2_pxi [get_bd_pins axi_gpio_pxi_0/gpio2_io_i] [get_bd_pins t_gpio_interface_0/gpio2_pxi]
  connect_bd_net -net t_gpio_interface_0_gpio_i_pxi [get_bd_pins axi_gpio_pxi_0/gpio_io_i] [get_bd_pins t_gpio_interface_0/gpio_i_pxi]
  connect_bd_net -net t_gpio_interface_0_gpio_pxi_star [get_bd_pins axi_gpio_pxi_star_0/gpio_io_i] [get_bd_pins t_gpio_interface_0/gpio_pxi_star]
  connect_bd_net -net t_gpio_interface_0_test [get_bd_ports test] [get_bd_pins t_gpio_interface_0/test]
  connect_bd_net -net t_gpio_interface_0_trig_oe [get_bd_ports trig_oe] [get_bd_pins t_gpio_interface_0/trig_oe]
  connect_bd_net -net t_gpio_interface_0_trig_out [get_bd_ports trig_out] [get_bd_pins t_gpio_interface_0/trig_out]
  connect_bd_net -net t_gpio_interface_0_trig_tsel [get_bd_ports trig_tsel] [get_bd_pins t_gpio_interface_0/trig_tsel]
  connect_bd_net -net trig_in_0_1 [get_bd_ports trig_in] [get_bd_pins t_gpio_interface_0/trig_in]
  connect_bd_net -net xlconcat_0_dout [get_bd_pins axi_intc_0/intr] [get_bd_pins xlconcat_0/dout]

  # Create address segments
  create_bd_addr_seg -range 0x00020000 -offset 0x00000000 [get_bd_addr_spaces microblaze_0/Data] [get_bd_addr_segs axi_bram_ctrl_0/S_AXI/Mem0] SEG_axi_bram_ctrl_0_Mem0
  create_bd_addr_seg -range 0x00020000 -offset 0x00000000 [get_bd_addr_spaces microblaze_0/Instruction] [get_bd_addr_segs axi_bram_ctrl_0/S_AXI/Mem0] SEG_axi_bram_ctrl_0_Mem0
  create_bd_addr_seg -range 0x00010000 -offset 0x40000000 [get_bd_addr_spaces microblaze_0/Data] [get_bd_addr_segs axi_gpio_cex_0/S_AXI/Reg] SEG_axi_gpio_0_Reg
  create_bd_addr_seg -range 0x00010000 -offset 0x40010000 [get_bd_addr_spaces microblaze_0/Data] [get_bd_addr_segs axi_gpio_pxi_star_0/S_AXI/Reg] SEG_axi_gpio_count_0_Reg
  create_bd_addr_seg -range 0x00010000 -offset 0x40020000 [get_bd_addr_spaces microblaze_0/Data] [get_bd_addr_segs axi_gpio_pxi_0/S_AXI/Reg] SEG_axi_gpio_pxi_0_Reg
  create_bd_addr_seg -range 0x00010000 -offset 0x41200000 [get_bd_addr_spaces microblaze_0/Data] [get_bd_addr_segs axi_intc_0/S_AXI/Reg] SEG_axi_intc_0_Reg
  create_bd_addr_seg -range 0x00010000 -offset 0x41A00000 [get_bd_addr_spaces microblaze_0/Data] [get_bd_addr_segs axi_timebase_wdt_0/S_AXI/Reg] SEG_axi_timebase_wdt_0_Reg
  create_bd_addr_seg -range 0x00010000 -offset 0x41C00000 [get_bd_addr_spaces microblaze_0/Data] [get_bd_addr_segs axi_timer_0/S_AXI/Reg] SEG_axi_timer_0_Reg
  create_bd_addr_seg -range 0x00010000 -offset 0x40600000 [get_bd_addr_spaces microblaze_0/Data] [get_bd_addr_segs axi_uartlite_0/S_AXI/Reg] SEG_axi_uartlite_0_Reg


  # Restore current instance
  current_bd_instance $oldCurInst

  save_bd_design
}
# End of create_root_design()


##################################################################
# MAIN FLOW
##################################################################

create_root_design ""


