#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers

from Item import *
from COMe_Serial import *


"""
Interface to PXCT via COMe serial interface
"""

class PXCT(COMe_Serial):

    _parameters = {'device':['Serial device', 'Serial device used to communicate', "/dev/ttyS0", 'file'],
                  'bauds':['Bauds', 'Speed of the communication', 115200, int],
                  'to':['Timeout', 'Timeout during read operations',  1, int]}

    def __init__(self, *args, **kwargs):
        """The initializer doesn't connect to the device."""
        COMe_Serial.__init__(self, *args, **kwargs)

    def connect(self):
        COMe_Serial.connect(self)
        self.attention()

    # attention
    def attention(self):
        return self.command("!")

    # pxid: PXI_TRIG direction (pxi_trig[7:0]) (0:output, 1:input)
    def set_pxi_trig_direction(self, direction=0xFF):
        return self.command(":PXID=0x%x" % (direction))

    def get_pxi_trig_direction(self):
        return self.command(":PXID?")

    # pxi: PXI_TRIG (pxi_trig[7:0])
    def set_pxi_trig(self, value):
        return self.command(":PXI=0x%x" % (value))

    def get_pxi_trig(self):
        return self.command(":PXI?")

    # lbld: PXI_LBL6 direction (pxi_lbl6) (0:output, 1:input)
    def set_pxi_lbl6_direction(self, direction=0x1):
        return self.command(":LBLD=0x%x" % (direction))

    def get_pxi_lbl6_direction(self):
        return self.command(":LBLD?")

    # lbl: PXI_LBL6 (pxi_lbl6)
    def set_pxi_lbl6(self, value):
        return self.command(":LBL=0x%x" % (value))

    def get_pxi_lbl6(self):
        return self.command(":LBL?")

    # lbrd: PXI_LBR6 direction (pxi_lbr6) (0:output, 1:input)
    def set_pxi_lbr6_direction(self, direction=0x1):
        return self.command(":LBRD=0x%x" % (direction))

    def get_pxi_lbr6_direction(self):
        return self.command(":LBRD?")

    # lbr: PXI_LBR6 (pxi_lbr6)
    def set_pxi_lbr6(self, value):
        return self.command(":LBR=0x%x" % (value))

    def get_pxi_lbr6(self):
        return self.command(":LBR?")

    # ga: PXI_GA (pxi_ga[4:0])
    def get_pxi_ga(self):
        return self.command(":GA?")
    
    # star: PXI_STAR (star_out)
    def set_pxi_star(self, value):
        return self.command(":STAR=0x%x" % (value))

    # star: PXI_STAR (star_in)
    def get_pxi_star(self):
        return self.command(":STAR?")
    
    # stars: PXI_STAR select (star_sel) (0:star_out, 1:pxi_clk10)
    def set_pxi_star_select(self, value):
        return self.command(":STARS=0x%x" % (value))
    
    def get_pxi_star_select(self):
        return self.command(":STARS?")

    # stare: PXI_STAR enable (star_en, dstar_en)
    def set_pxi_star_enable(self, value):
        return self.command(":STARE=0x%x" % (value))
    
    def get_pxi_star_enable(self):
        return self.command(":STARE?")

    # dstar: PXIe_DSTARA (dstar_out)
    def set_pxie_dstar(self, value):
        return self.command(":DSTAR=0x%x" % (value))

    # dstars: PXIe_DSTARA select (dstar_sel) (0:dstar_out, 1:pxi_clk10)
    def set_pxie_dstar_select(self, value):
        return self.command(":DSTARS=0x%x" % (value))
    
    def get_pxie_dstar_select(self):
        return self.command(":DSTARS?")

    # trgd: TRIG direction (trig_oe) (0:output, 1:input)
    def set_trig_direction(self, direction=0x1):
        return self.command(":TRGD=0x%x" % (direction))

    def get_trig_direction(self):
        return self.command(":TRGD?")

    # trgt: TRIG 50 Ohm termination (trig_tsel)
    def set_trig_termination(self, termination):
        return self.command(":TRGT=0x%x" % (termination))

    def get_trig_termination(self):
        return self.command(":TRGT?")

    # trg: TRIG (trig_in/trig_out)
    def set_trig(self, value):
        return self.command(":TRG=0x%x" % (value))

    def get_trig(self):
        return self.command(":TRG?")

    # trgl: TRIG ADC level (trig_in/trig_out)
    def get_trig_level(self):
        return self.command(":TRGL?")


if __name__ == "__main__":
    main()
    
        

