#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers
#------------------------------------------------------------------------
# Changes
# 2022-11-14 INCAA Computers
#   Added import of ptspxiecome
#   Added COM1_DEVICE as device
#------------------------------------------------------------------------

from ptspxiecome import *
from COMe_Serial import *

"""
Interface to PXIeCOMe tFPGA via COMe serial interface
"""

COM1_DEVICE = COMe_module_com1_device()

class PXIeCOMe_tFPGA(COMe_Serial):

    def __init__(self):
        """The initializer doesn't connect to the device."""
        COMe_Serial.__init__(self, device=COM1_DEVICE, bauds=115200)


    # pxid: PXI_TRIG direction (pxi_trig7_t[2:0], pxi_trig_t[7:0]) (0:output, 1:input)
    def set_pxi_trig_direction(self, direction=0x7FF):
        return self.command(":PXID=0x%x" % (direction))

    def get_pxi_trig_direction(self):
        return self.command(":PXID?")

    # pxi: PXI_TRIG (pxi_trig7_i/o[2:0], pxi_trig_i/o[7:0])
    def set_pxi_trig(self, value):
        return self.command(":PXI=0x%x" % (value))

    def get_pxi_trig(self):
        return self.command(":PXI?")

    # lbrd: PXI_LBR6 direction (pxi_lbr6_t) (0:output, 1:input)
    def set_pxi_lbr6_direction(self, direction=0x1):
        return self.command(":LBRD=0x%x" % (direction))

    def get_pxi_lbr6_direction(self):
        return self.command(":LBRD?")

    # lbr: PXI_LBR6 (pxi_lbr6_i/o)
    def set_pxi_lbr6(self, value):
        return self.command(":LBR=0x%x" % (value))

    def get_pxi_lbr6(self):
        return self.command(":LBR?")

    # ga: PXI_GA (pxi_ga[4:0])
    def get_pxi_ga(self):
        return self.command(":GA?")
    
    # star: PXI_STAR (pxi_star)
    def get_pxi_star(self):
        return self.command(":STAR?")
    
    # starr: PXI_STAR recorded (pxi_star)
    def get_pxi_star_rec(self):
        return self.command(":STARR?")

    # trgd: TRIG direction (trig_oe) (0:output, 1:input)
    def set_trig_direction(self, direction=0x1):
        return self.command(":TRGD=0x%x" % (direction))

    def get_trig_direction(self):
        return self.command(":TRGD?")

    # trgt: TRIG 50 Ohm termination (trig_tsel)
    def set_trig_termination(self, termination):
        return self.command(":TRGT=0x%x" % (termination))

    def get_trig_termination(self):
        return self.command(":TRGT?")

    # trg: TRIG (trig_in/trig_out)
    def set_trig(self, value):
        return self.command(":TRG=0x%x" % (value))

    def get_trig(self):
        return self.command(":TRG?")

    # gpo: GPO (fpga_gpo[1:0] -> cex_gpi[1:0])
    def set_gpo(self, value):
        return self.command(":GPO=0x%x" % (value))

    def get_gpo(self):
        return self.command(":GPO?")

    # gpoe: GPO Select (fpga_gpo_sel_N: enable fpga_gpo[1:0] -> cex_gpi[1:0])
    def set_gpo_select(self, value):
        return self.command(":GPOE=0x%x" % (value))

    def get_gpo_select(self):
        return self.command(":GPOE?")

    # gpi: GPI (fpga_gpi[3:0] <- cex_gpo[3:0])
    def get_gpi(self):
        return self.command(":GPI?")


if __name__ == "__main__":
    main()
    
        

