#!/usr/bin/python

import argparse, sys
import os


def parseArguments():
    parser=argparse.ArgumentParser()

    parser.add_argument('-tool', help='vivado/vivado_lab/program_flash\n\tDefault Value : Vivado',default="vivado")
    parser.add_argument('-url', help='Add hw_server URL\n\tDefault Value : localhost:3121',default="localhost:3121")
    parser.add_argument('-flash', help='Name of flash to be programmed\n\tDefault Value : mx25u51245g-spi-x1_x2_x4_x8',default="mx25u51245g-spi-x1_x2_x4_x8")
    parser.add_argument('-mcs_file', help='Path for the MCS file')
    parser.add_argument('-sec_file', help='Path for Secondary MCS File incase of spi_x8 flash\n\tDefault Value : None',default=None)
    parser.add_argument('-addr_range', help='Address Range : use_file,entire_configurations\n\tDefault Value : use_file',default="use_file")
    parser.add_argument('-erase', help='Erase Operation : 1/0\n\tDefault Value : 1',default="1")
    parser.add_argument('-blank_check', help='Blank Check Operation : 1/0\n\tDefault Value : 0',default="0")
    parser.add_argument('-program', help='Program Operation : 1/0\n\tDefault Value : 1',default="1")
    parser.add_argument('-verify', help='Verify Operation : 1/0\n\tDefault Value : 1',default="1")
    parser.add_argument('-frequency', help='JTAG Frequency \n\tDefault Value : 1500000 ',default="1500000")

    args=parser.parse_args()
    
    return args


def createVivadocmd(args):
    
    if args.mcs_file is None:
        print "Error: MCS FIle Path Not specified"
        __exec_cmd = None
    else:
        __exec_cmd = args.tool + " -mode batch -source vivado_flash.tcl -tclargs -url " + args.url + " -flash " + args.flash + " -mcs_file " + str(args.mcs_file) + " -addr_range " + args.addr_range + " -erase " + args.erase + " -blank_check " + args.blank_check + " -program " + args.program + " -verify " + args.verify
        if "spi_x1_x2_x4_x8" in args.flash:
            if args.sec_file is None:
                print "Error:  Secondary MCS FIle Path Not specified"
                __exec_cmd = None
            else:
                __exec_cmd = __exec_cmd + "-sec_file" + args.sec_file

    return __exec_cmd

            
        



def createProgFlashcmd(args):
      
      
    if args.mcs_file is None:
        print "Error: MCS FIle Path Not specified"
        __exec_cmd = None
    else:
        __exec_cmd = args.tool
        if "spi_x1_x2_x4_x8" in args.flash:
            if args.sec_file is None:
                print "Error:  Secondary MCS FIle Path Not specified"
                __exec_cmd = None
            else:
                __exec_cmd = __exec_cmd + " -f \"" + args.mcs_file + "\" \""+ args.sec_file + "\""
        else:
            __exec_cmd = __exec_cmd + " -f \"" + args.mcs_file + "\" "
            
        __exec_cmd = __exec_cmd + "-flash_type " + args.flash
        if args.erase == "0":
            __exec_cmd = __exec_cmd + " -no_erase "
        if args.blank_check == "1":
            __exec_cmd = __exec_cmd + " -blank_check "
        if args.verify == "1":
            __exec_cmd = __exec_cmd + " -verify "
        __exec_cmd = __exec_cmd + "-cable type xilinx_tcf url tcp:" + args.url + " frequency "+ args.freq 

    return __exec_cmd


if __name__ == '__main__':
    # Parse the arguments
    args = parseArguments()

    # Raw print arguments
    if args.tool == "vivado" or args.tool == "vivado_lab":

        __exec_cmd = createVivadocmd(args)
        if __exec_cmd is None:
            print ("Error found while processing, please check the previous errors")
        else:
            os.system (__exec_cmd)
    elif args.tool == "program_flash":
        __exec_cmd = createProgFlashcmd(args)
        if __exec_cmd is None:
            print ("Error found while processing, please check the previous errors")
        else:
            os.system (__exec_cmd)
    else:
        print ("Error: Invalid Tool Specified")

