#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers
#------------------------------------------------------------------------
# Changes
# 2022-11-14 INCAA Computers
#   Changed board_name to product_name, board_version to product_version,
#   board_serial to product_serial and board_asset_tag to product_sku
#------------------------------------------------------------------------

import sys
import time
import os
import subprocess

# Add common modules and libraries location to path
sys.path.append('../../../')

from ptsexcept import *
from ptspxiecome import *

"""
test00: Basic test setup
"""

setup_image = [
"+-----------PXIeCOMe--------------+--PXCT--+",
"|           LAN     USB           |        | CONNECT PXIeCOMe interfaces:",
"| 10/100 +------+ +-+ +-+         |        | -LAN (RJ45) to local network",
"|  /1000 |o     | | | | | o PWR   | o      | -USB6 to mouse",
"|        |o     | |6| |5| o DRIVE | o STAT | -USB5 to keyboard",
"|   ACT/ +------+ +-+ +-+         |        | -USB4 to USB-Serial adapter",
"|   LINK          +-+ +-+         |        | -USB3 to Xilinx Platform Cable",
"|                 | | | | USB     | /--\   | -USB2 to USB3.0 memory stick",
"|          /--\   |4| |3|         | |  |   | -USB1 to USB3.0 memory stick",
"|   TRIG   |  |   +-+ +-+         | \--/   | -DP to display",
"|          \--/   +-+ +-+         |   TRIG | -TRIG to PXCT TRIG port",
"|                 | | | | USB (SS)|        | -COM to PXCT COM port",
"|         +---\   |2| |1|         |  /---+ | ",
"|         |   |   +-+ +-+         |  |   | | Connect Xilinx Platform Cable",
"|    COM  |   |       +-+         |  |   | | to PXIeCOMe J3 (JTAG) through",
"|         |   |       | |         |  |   | | the opening in the PXCT PCB.",
"|         +---/       | | DP      |  \---+ | ",
"|                     | |         |   COM  | The 2nd PXCT should be in PXIe",
"|                     +-/         |        | System Timing slot. Connect this",
"+------------------------+-Slot 1-+-Slot 2-+ PXCT COM port to USB4 Serial."
]

tmp_stdout = sys.stdout
tmp_stdin = sys.stdin

def stdio_redirect_start():
    tmp_stdout = sys.stdout
    sys.stdout = sys.__stdout__
    tmp_stdin = sys.stdin
    sys.stdin = sys.__stdin__

def stdio_redirect_end():
    sys.stdout = tmp_stdout
    sys.stdin = tmp_stdin


def main (default_directory='.'):

    print "Test00 (Setup) Start"
    init_test_time = time.time()
    ret_error = None
    abort = None

    stdio_redirect_start()
    print ("")
    for i in range(len(setup_image)):
        print (" %s") % setup_image[i]
    print ("")
    ask = "";
    while ((ask != "Y") and (ask != "N") and (ask != "A")):
        print "-------------------------------------------------------------"
        ask = raw_input("Has the test setup been connected as shown above? [Y(es),N(o),A(bort)]")
        ask = ask.upper()
        print " "

    stdio_redirect_end()
    if (ask == "A"):
        abort = "Test aborted"
    if (ask == "N"):
        ret_error = 'Incorrect setup.'
    if abort:
        raise PtsCritical(abort)

    print "COMe Module used within the setup:"
    print "  Vendor: " + COMe_module_attrib('board_vendor')
    print "  Name: " + COMe_module_attrib('product_name')
    print "  Version: " + COMe_module_attrib('product_version')
    print "  Serial: " + COMe_module_attrib('product_serial')
    print "  Partnumber: " + COMe_module_attrib('product_sku')
    print "  BIOS vendor: " + COMe_module_attrib('bios_vendor')
    print "  BIOS version: " + COMe_module_attrib('bios_version')
    print "  BIOS date: " + COMe_module_attrib('bios_date')
    
    end_test_time = time.time()
    print "End of Test00 (Setup)"
    print "RESULT: [{}]".format("FAIL" if ret_error else "OK")
    print 'Test00 elapsed time: {:.2f} seconds'.format(end_test_time-init_test_time)
    if ret_error:
        raise PtsError(ret_error)
    

if __name__ == '__main__' :
    main()
