#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers

import sys
import time
import os
import subprocess

# Add common modules and libraries location to path
sys.path.append('../../../')

from ptsexcept import *
from ptspxiecome import *

"""
test02: Check DisplayPort

Since the PTS must be started while using a display, mouse and keyboard, the
fact that the PTS test can be started while using the display makes it possible
to check the DiplayPort.
In case of problems the PTS CentOS system could also be remotely accessed
using the LAN interface and Remote Desktop
This test logs several DisplayPort and display aspects.
When using a DisplayPort to HDMI or DVI converter cable this test could fail
because it detects a HDMI display instead of a DisplayPort display.
"""

def dp_port_enabled(card):
    sysfs_file = '/sys/class/drm/%s/enabled' % (card)
    return shell_cat_file(sysfs_file)

def dp_port_which_enabled():
    dp = 'card0-DP-1'
    hdmi = 'card0-HDMI-A-1'
    if (dp_port_enabled(dp) == 'enabled'):
        return dp, True
    else:
        if (dp_port_enabled(hdmi) == 'enabled'):
            return hdmi, False
    return 'None', False

def dp_port_status():
    card, realDP = dp_port_which_enabled()
    sysfs_file = '/sys/class/drm/%s/status' % (card)
    return shell_cat_file(sysfs_file)

def dp_port_modes():
    card, realDP = dp_port_which_enabled()
    sysfs_file = '/sys/class/drm/%s/modes' % (card)
    return shell_cat_file(sysfs_file)

def dp_port_edid():
    card, realDP = dp_port_which_enabled()
    sysfs_file = '/sys/class/drm/%s/edid' % (card)
    if os.path.exists(sysfs_file):
        sysfs_cmd = "hexdump -C " + sysfs_file
        df = subprocess.check_output(sysfs_cmd, shell=True)
        return df.rstrip()
    return "None"

def main (default_directory='.'):

    print "Test02 (DisplayPort) Start"
    init_test_time = time.time()
    ret_error = None

    card, realDP = dp_port_which_enabled()
    # list DisplayPort related information
    print ""
    print "Card: %s %s" % (card, dp_port_status()),
    if not realDP:
        ret_error = "Not connected to a DisplayPort display"
        print_error()
    else:
        print_ok()
    print ""
    print "Display EDID:\n%s\n" % (dp_port_edid())
    print "Display Modes:\n%s\n" % (dp_port_modes())

    end_test_time = time.time()
    print "End of Test02 (DisplayPort)"
    print "RESULT: [{}]".format("FAIL" if ret_error else "OK")
    print 'Test02 elapsed time: {:.2f} seconds'.format(end_test_time-init_test_time)
    if ret_error:
        raise PtsError(ret_error)
    

if __name__ == '__main__' :
    main()
