#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers

import sys
import time
import os
import subprocess

# Add common modules and libraries location to path
sys.path.append('../../../')

from ptsexcept import *
from ptspxiecome import *

"""
test03: Check LAN

Check aspects of the LAN interface and perform a simple ping to a remote server.
This test also logs several LAN aspects and tests the LAN RJ45 LEDS
"""

NET_DEVICE = COMe_module_net_device()

def lan_check_ping():
    hostname = "cern.org"
    cmd = 'ping -c 1 %s' % (hostname)
    try:
        df = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
        output = df.rstrip()
        status = 0
    except subprocess.CalledProcessError as e:
        output = e.output.rstrip()
        status = e.returncode
    print "%s" % (output)
    return status

def lan_operstate():
    sysfs_file = '/sys/class/net/%s/operstate' % (NET_DEVICE)
    return shell_cat_file(sysfs_file)

def lan_speed():
    sysfs_file = '/sys/class/net/%s/speed' % (NET_DEVICE)
    return shell_cat_file(sysfs_file)

def lan_duplex():
    sysfs_file = '/sys/class/net/%s/duplex' % (NET_DEVICE)
    return shell_cat_file(sysfs_file)

def lan_mac_address():
    sysfs_file = '/sys/class/net/%s/address' % (NET_DEVICE)
    return shell_cat_file(sysfs_file)

tmp_stdout = sys.stdout
tmp_stdin = sys.stdin

def stdio_redirect_start():
    tmp_stdout = sys.stdout
    sys.stdout = sys.__stdout__
    tmp_stdin = sys.stdin
    sys.stdin = sys.__stdin__

def stdio_redirect_end():
    sys.stdout = tmp_stdout
    sys.stdin = tmp_stdin


def main (default_directory='.'):

    print "Test03 (LAN) Start"
    init_test_time = time.time()
    ret_error = None
    abort = None

    stdio_redirect_start()
    ask = "";
    while ((ask != "Y") and (ask != "N") and (ask != "A")):
        print "-------------------------------------------------------------"
        ask = raw_input("Is the LAN LED (10/100/1000) switched on? [Y(es),N(o),A(bort)]")
        ask = ask.upper()
        print " "
    stdio_redirect_end()
    if (ask == "A"):
        abort = "Test aborted"
    else:
        print 'LED (10/100/1000)',
        if (ask == "N"):
            ret_error = 'LED (10/100/1000) not functioning.'
            print_error()
        if (ask == "Y"):
            print_ok()
    if abort:
        raise PtsCritical(abort)

    stdio_redirect_start()
    ask = "";
    while ((ask != "Y") and (ask != "N") and (ask != "A")):
        print "-------------------------------------------------------------"
        ask = raw_input("Is the LAN LED (ACT/LINK) switched on or blinking? [Y(es),N(o),A(bort)]")
        ask = ask.upper()
        print " "
    stdio_redirect_end()
    if (ask == "A"):
        abort = "Test aborted"
    else:
        print 'LED (ACT/LINK)',
        if (ask == "N"):
            ret_error = 'LED (ACT/LINK) not functioning.'
            print_error()
        if (ask == "Y"):
            print_ok()
    if abort:
        raise PtsCritical(abort)
    
    # Check LAN status
    status = lan_check_ping()
    print "LAN ping status",
    if status != 0:
        ret_error = 'LAN ping status: %d' % (status)
        print_error()
    else:
        print_ok()
    print ""

    # list ATA related information
    operstate = lan_operstate()
    print "LAN state: %s" % (operstate),
    if operstate != 'up':
        ret_error = ('LAN interface not up, but %s.' % (operstate))
        print_error()
    else:
        print_ok()
    print ""

    speed = lan_speed()
    print "LAN speed: %s" % (speed),
    if speed != '1000':
        ret_error = ('LAN interface not at 1000 Mbps, but at %s Mbps.' % (speed))
        print_error()
    else:
        print_ok()
    print ""

    duplex = lan_duplex()
    print "LAN duplex: %s" % (duplex),
    if duplex != 'full':
        ret_error = ('LAN interface not full duplex, but %s duplex.' % (duplex))
        print_error()
    else:
        print_ok()
    print ""

    print "MAC address: %s\n" % (lan_mac_address())

    end_test_time = time.time()
    print "End of Test03 (LAN)"
    print "RESULT: [{}]".format("FAIL" if ret_error else "OK")
    print 'Test03 elapsed time: {:.2f} seconds'.format(end_test_time-init_test_time)
    if ret_error:
        raise PtsError(ret_error)
        

if __name__ == '__main__' :
    main()
