#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers
#------------------------------------------------------------------------
# Changes
# 2022-11-14 INCAA Computers
#   Added COM0_DEVICE to PXCT()
#   Added VCOM_DEVICE to PXCT()
#------------------------------------------------------------------------

import sys
import time
import os
import subprocess

# Add common modules and libraries location to path
sys.path.append('../../../')

from ptsexcept import *
from ptspxiecome import *

from PXIeCOMe_tFPGA import * 
from PXCT import * 

"""
test06: Check Serial (COM)

Check the serial communication of the front panel COM port (SER0) and the
serial interface to the FPGA (SER1).
"""

COM0_DEVICE = COMe_module_com0_device()
VCOM_DEVICE = '/dev/ttyUSB0'

def main (default_directory='.'):

    print "Test06 (COM) Start"
    init_test_time = time.time()
    ret_error = None

    # Front panel COM port connected to the PXCT module in slot 2
    print ""
    com_PXCT_p = PXCT(device = COM0_DEVICE)
    com_PXCT_p.connect()
    response = com_PXCT_p.id()
    print 'PXCT (SER0) ID response: %s' % (response),
    exp_response = 'INCAA Computers,CPE-PXCT'
    if not response.startswith(exp_response):
        ret_error = 'PXCT (SER0) incorrect response.'
        print_error()
    else:
        print_ok()
    com_PXCT_p.close()

    # Front panel USB4 port connected USB-Serial adapter and to the
    # PXCT module in System Timing slot
    print ""
    com_PXCT_t = PXCT(device = VCOM_DEVICE)
    com_PXCT_t.connect()
    response = com_PXCT_t.id()
    print 'PXCT (USB) ID response: %s' % (response),
    exp_response = 'INCAA Computers,CPE-PXCT'
    if not response.startswith(exp_response):
        ret_error = 'PXCT (USB) incorrect response.'
        print_error()
    else:
        print_ok()
    com_PXCT_t.close()

    # Internal serial port connected to the onboard FPGA
    print ""
    com_tFPGA = PXIeCOMe_tFPGA()
    com_tFPGA.connect()
    response = com_tFPGA.id()
    print 'FPGA (SER1) ID response: %s' % (response),
    exp_response = 'INCAA Computers,PXIeCOMe'
    if not response.startswith(exp_response):
        ret_error = 'FPGA (SER1) incorrect response.'
        print_error()
    else:
        print_ok()
    com_tFPGA.close()

    print ""
    end_test_time = time.time()
    print "End of Test06 (COM)"
    print "RESULT: [{}]".format("FAIL" if ret_error else "OK")
    print 'Test06 elapsed time: {:.2f} seconds'.format(end_test_time-init_test_time)
    if ret_error:
        raise PtsError(ret_error)
    

if __name__ == '__main__' :
    main()
