#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers
#------------------------------------------------------------------------
# Changes
# 2022-11-15 INCAA Computers
#   Moved documentation to ptspxiecome.py
# 2022-12-15 INCAA Computers
#   Removed GPIO_BASE_ADDR (base addr of the gpio is now set dynamically)
#   Added GPIOCHIP_LABEL
#   Added print of the used gpiochip
#------------------------------------------------------------------------

import sys
import time
import os
import subprocess

# Add common modules and libraries location to path
sys.path.append('../../../')

from ptsexcept import *
from ptspxiecome import *

from COMe_GPIO import *
from PXIeCOMe_tFPGA import * 

"""
test07: Check GPIO

Check the COM Express GPI an GPO signals.
"""

GPIOCHIP_LABEL = COMe_module_gpiochip_label()

GPI0 = 0
GPI1 = 1
GPI2 = 2
GPI3 = 3

SYSEN_N = GPI0
LINKCAP = GPI1

GPO0 = 0x01
GPO1 = 0x02
GPO2 = 0x04
GPO3 = 0x08

tmp_stdout = sys.stdout
tmp_stdin = sys.stdin

def stdio_redirect_start():
    tmp_stdout = sys.stdout
    sys.stdout = sys.__stdout__
    tmp_stdin = sys.stdin
    sys.stdin = sys.__stdin__

def stdio_redirect_end():
    sys.stdout = tmp_stdout
    sys.stdin = tmp_stdin


def main (default_directory='.'):

    print "Test07 (GPIO) Start"
    init_test_time = time.time()
    ret_error = None
    abort = None

    # GPIO
    print ""
    come_gpio = COMe_GPIO(label=GPIOCHIP_LABEL)
    come_gpio.setup()
    print 'Using GPIO %s via gpiochip%d' % (GPIOCHIP_LABEL, come_gpio.baseaddr())
    print ""

    # Internal serial port connected to the onboard FPGA
    tFPGA = PXIeCOMe_tFPGA()
    tFPGA.connect()
    response = tFPGA.id()
    print 'FPGA ID response: %s' % (response)
    print ""

    # GPO0 to GPO3
    for gpo in range(0, 4):
        mask = 0x1 << gpo
        for stim in range(0, 2, 1):
            come_gpio.gpo(gpo, stim)
            val = tFPGA.get_gpi()
            val &= mask
            val >>= gpo
            print "GPO%d = %d" % (gpo, val),
            if (val != stim):
                ret_error = "GPO%d incorrect. Expected '%d'." % (gpo, stim)
                print_error()
            else:
                print_ok()
        for stim in range(1, -1, -1):
            come_gpio.gpo(gpo, stim)
            val = tFPGA.get_gpi()
            val &= mask
            val >>= gpo
            print "GPO%d = %d" % (gpo, val),
            if (val != stim):
                ret_error = "GPO%d incorrect. Expected '%d'." % (gpo, stim)
                print_error()
            else:
                print_ok()
        print ""
    
    # GPI0
    tFPGA.set_gpo_select(0)
    val = come_gpio.gpi(SYSEN_N)
    print "SYSEN_N = %d" % (val),
    if (val != 0):
        ret_error = "GPI0 (SYSEN_N) incorrect. Expected '0'."
        print_error()
    else:
        print_ok()
    tFPGA.set_gpo_select(1)
    tFPGA.set_gpo(GPO0)
    val = come_gpio.gpi(GPI0)
    print "GPI0 = %d" % (val),
    if (val != 1):
        ret_error = "GPI0 incorrect. Expected '1'."
        print_error()
    else:
        print_ok()
    tFPGA.set_gpo(0)
    val = come_gpio.gpi(GPI0)
    print "GPI0 = %d" % (val),
    if (val != 0):
        ret_error = "GPI0 incorrect. Expected '0'."
        print_error()
    else:
        print_ok()
    print ""

    # GPI1
    tFPGA.set_gpo_select(0)
    val = come_gpio.gpi(LINKCAP)
    print "LINKCAP = %d" % (val),
    if (val != 1):
        ret_error = 'GPI1 (LINKCAP) incorrect.'
        print_error()
    else:
        print_ok()
    tFPGA.set_gpo_select(1)
    tFPGA.set_gpo(GPO1)
    val = come_gpio.gpi(GPI1)
    print "GPI1 = %d" % (val),
    if (val != 1):
        ret_error = "GPI1 incorrect. Expected '1'."
        print_error()
    else:
        print_ok()
    tFPGA.set_gpo(0)
    val = come_gpio.gpi(GPI1)
    print "GPI1 = %d" % (val),
    if (val != 0):
        ret_error = "GPI1 incorrect. Expected '0'."
        print_error()
    else:
        print_ok()
    print ""

    tFPGA.close()

    stdio_redirect_start()
    ask = "";
    while ((ask != "Y") and (ask != "A")) :
        print "--------------------------------------------------------------"
        print "Please push the PROG button once during the next 5 second."
        ask = raw_input("Ready to start the timer? [Y(es),A(bort)]")
        ask = ask.upper()
        print " "
    stdio_redirect_end()
    if (ask == "A"):
        abort = "Test aborted"
    if abort:
        raise PtsCritical(abort)

    curtime = current_time()
    timeout = 5
    count = timeout
    gpi2 = [0] *2
    gpi3 = [0] *2
    while 1:
        count = timeout - (current_time() - curtime)
        if count < 0:
            count = 0
        stdio_redirect_start()
        print "%d\r" % (count),
        stdio_redirect_end()
        if (current_time() - curtime) > timeout:
            break

        # GPI2 (INIT_N)
        val = come_gpio.gpi(GPI2)
        if (val == 0 or val == 1):
            if (gpi2[val] != 1):
                print "GPI2 = %d" % (val)
                gpi2[val] = 1

        # GPI3 (DONE)
        val = come_gpio.gpi(GPI3)
        if (val == 0 or val == 1):
            if (gpi3[val] != 1):
                print "GPI3 = %d" % (val)
                gpi3[val] = 1
    stdio_redirect_start()
    print ""
    stdio_redirect_end()
            
    print " "
    # GPI2
    print "GPI2",
    if (gpi2[0] != 1 or gpi2[1] != 1):
        print_error()
    else:
        print_ok()
    if (gpi2[0] != 1):
        ret_error = "GPI2 incorrect. Expected '0'."
    if (gpi2[1] != 1):
        ret_error = "GPI2 incorrect. Expected '1'."

    # GPI3
    print "GPI3",
    if (gpi3[0] != 1 or gpi3[1] != 1):
        print_error()
    else:
        print_ok()
    if (gpi3[0] != 1):
        ret_error = "GPI3 incorrect. Expected '0'."
    if (gpi3[1] != 1):
        ret_error = "GPI3 incorrect. Expected '1'."

    come_gpio.cleanup()

    print ""
    end_test_time = time.time()
    print "End of Test07 (GPIO)"
    print "RESULT: [{}]".format("FAIL" if ret_error else "OK")
    print 'Test07 elapsed time: {:.2f} seconds'.format(end_test_time-init_test_time)
    if ret_error:
        raise PtsError(ret_error)
    

if __name__ == '__main__' :
    main()
