#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2021-2022
# This source describes Open Hardware and is licensed under the
# CERN-OHL-W-2.0+
# Source location: https://ohwr.org/project/pxie-ctl-comexpress-tst
# designed by INCAA Computers
#------------------------------------------------------------------------
# Changes
# 2022-11-15 INCAA Computers
#   Added COM0_DEVICE to PXCT()
#   Added VCOM_DEVICE to PXCT()
#------------------------------------------------------------------------

import sys
import time
import os
import subprocess

# Add common modules and libraries location to path
sys.path.append('../../../')

from ptsexcept import *
from ptspxiecome import *

from PXIeCOMe_tFPGA import * 
from PXCT import * 

"""
test10: Check PXI

Check the interface to the PXI backplane using two PXCT modules and check the
PXI_TRIG[7:0], PXI_LBR6, PXI_GA[4:0] and PXI_STAR signals.
"""

COM0_DEVICE = COMe_module_com0_device()
VCOM_DEVICE = '/dev/ttyUSB0'

def pxi_init(com_tFPGA, com_PXCT):
    # initialize all interface directions to input
    com_tFPGA.set_pxi_trig_direction()
    com_tFPGA.set_pxi_lbr6_direction()
    com_PXCT.set_pxi_trig_direction()
    com_PXCT.set_pxi_lbl6_direction()
    com_PXCT.set_pxi_lbr6_direction()

pxi_trig_write_test = [
    { 'write':0x000, 'read':0x00, 'direction':0x700},
    { 'write':0x001, 'read':0x01, 'direction':0x700},
    { 'write':0x002, 'read':0x02, 'direction':0x700},
    { 'write':0x004, 'read':0x04, 'direction':0x700},
    { 'write':0x008, 'read':0x08, 'direction':0x700},
    { 'write':0x010, 'read':0x10, 'direction':0x700},
    { 'write':0x020, 'read':0x20, 'direction':0x700},
    { 'write':0x040, 'read':0x40, 'direction':0x700},
    { 'write':0x080, 'read':0x80, 'direction':0x700},
    { 'write':0x0FF, 'read':0xFF, 'direction':0x700},
    { 'write':0x0FE, 'read':0xFE, 'direction':0x700},
    { 'write':0x0FD, 'read':0xFD, 'direction':0x700},
    { 'write':0x0FB, 'read':0xFB, 'direction':0x700},
    { 'write':0x0F7, 'read':0xF7, 'direction':0x700},
    { 'write':0x0EF, 'read':0xEF, 'direction':0x700},
    { 'write':0x0DF, 'read':0xDF, 'direction':0x700},
    { 'write':0x0BF, 'read':0xBF, 'direction':0x700},
    { 'write':0x07F, 'read':0x7F, 'direction':0x700},
    { 'write':0x055, 'read':0x55, 'direction':0x700},
    { 'write':0x0AA, 'read':0xAA, 'direction':0x700},
    { 'write':0x033, 'read':0x33, 'direction':0x700},
    { 'write':0x0CC, 'read':0xCC, 'direction':0x700},
    { 'write':0x00F, 'read':0x0F, 'direction':0x700},
    { 'write':0x0F0, 'read':0xF0, 'direction':0x700},
    { 'write':0x0FF, 'read':0xFF, 'direction':0x700},
    { 'write':0x000, 'read':0x00, 'direction':0x700},
    { 'write':0x040, 'read':0x40, 'direction':0x700},
    { 'write':0x080, 'read':0x80, 'direction':0x700},
    { 'write':0x0C0, 'read':0xC0, 'direction':0x700},
    { 'write':0x000, 'read':0x00, 'direction':0x680},
    { 'write':0x040, 'read':0x40, 'direction':0x680},
    { 'write':0x100, 'read':0x80, 'direction':0x680},
    { 'write':0x140, 'read':0xC0, 'direction':0x680},
    { 'write':0x000, 'read':0x00, 'direction':0x580},
    { 'write':0x040, 'read':0x40, 'direction':0x580},
    { 'write':0x200, 'read':0x80, 'direction':0x580},
    { 'write':0x240, 'read':0xC0, 'direction':0x580},
    { 'write':0x000, 'read':0x00, 'direction':0x380},
    { 'write':0x040, 'read':0x40, 'direction':0x380},
    { 'write':0x400, 'read':0x80, 'direction':0x380},
    { 'write':0x440, 'read':0xC0, 'direction':0x380}
]

pxi_trig_read_test = [
    { 'write':0x00, 'read':0x000, 'direction':0x00},
    { 'write':0x01, 'read':0x001, 'direction':0x00},
    { 'write':0x02, 'read':0x002, 'direction':0x00},
    { 'write':0x04, 'read':0x004, 'direction':0x00},
    { 'write':0x08, 'read':0x008, 'direction':0x00},
    { 'write':0x10, 'read':0x010, 'direction':0x00},
    { 'write':0x20, 'read':0x020, 'direction':0x00},
    { 'write':0x40, 'read':0x040, 'direction':0x00},
    { 'write':0x80, 'read':0x780, 'direction':0x00},
    { 'write':0xFF, 'read':0x7FF, 'direction':0x00},
    { 'write':0xFE, 'read':0x7FE, 'direction':0x00},
    { 'write':0xFD, 'read':0x7FD, 'direction':0x00},
    { 'write':0xFB, 'read':0x7FB, 'direction':0x00},
    { 'write':0xF7, 'read':0x7F7, 'direction':0x00},
    { 'write':0xEF, 'read':0x7EF, 'direction':0x00},
    { 'write':0xDF, 'read':0x7DF, 'direction':0x00},
    { 'write':0xBF, 'read':0x7BF, 'direction':0x00},
    { 'write':0x7F, 'read':0x07F, 'direction':0x00},
    { 'write':0x55, 'read':0x055, 'direction':0x00},
    { 'write':0xAA, 'read':0x7AA, 'direction':0x00},
    { 'write':0x33, 'read':0x033, 'direction':0x00},
    { 'write':0xCC, 'read':0x7CC, 'direction':0x00},
    { 'write':0x0F, 'read':0x00F, 'direction':0x00},
    { 'write':0xF0, 'read':0x7F0, 'direction':0x00},
    { 'write':0xFF, 'read':0x7FF, 'direction':0x00}
]

def pxi_trig_test(com_tFPGA, com_PXCT):
    ret_error = None
    # PXI_TRIG write 
    pxi_init(com_tFPGA, com_PXCT)
    for test in pxi_trig_write_test:
        print "PXI_TRIG test write 0x%X" % (test['write']),
        com_tFPGA.set_pxi_trig_direction(test['direction'])
        com_tFPGA.set_pxi_trig(test['write'])
        val = com_PXCT.get_pxi_trig()
        if val != test['read']:
            ret_error = "Incorrect PXI_TRIG value read: 0x%X, expected: 0x%X" % (val, test['read'])
            print "(read: 0x%X, expected: 0x%X)" % (val, test['read']),
            print_error()
        else:
            print_ok()
    print ""
    # PXI_TRIG read
    pxi_init(com_tFPGA, com_PXCT)
    for test in pxi_trig_read_test:
        print "PXI_TRIG test read 0x%X" % (test['read']),
        com_PXCT.set_pxi_trig_direction(test['direction'])
        com_PXCT.set_pxi_trig(test['write'])
        val = com_tFPGA.get_pxi_trig()
        if val != test['read']:
            ret_error = "Incorrect PXI_TRIG value read: 0x%X, expected: 0x%X" % (val, test['read'])
            print "(read: 0x%X, expected: 0x%X)" % (val, test['read']),
            print_error()
        else:
            print_ok()
    print ""
    pxi_init(com_tFPGA, com_PXCT)
    return ret_error
    
pxi_lbr6_write_test = [
    { 'write':0x0, 'read':0x0, 'direction':0x0}, 
    { 'write':0x1, 'read':0x1, 'direction':0x0}, 
    { 'write':0x0, 'read':0x0, 'direction':0x0}, 
    { 'write':0x1, 'read':0x1, 'direction':0x0} 
]

pxi_lbr6_read_test = [
    { 'write':0x0, 'read':0x0, 'direction':0x0}, 
    { 'write':0x1, 'read':0x1, 'direction':0x0},
    { 'write':0x0, 'read':0x0, 'direction':0x0}, 
    { 'write':0x1, 'read':0x1, 'direction':0x0}
]

def pxi_lbr6_test(com_tFPGA, com_PXCT):
    ret_error = None
    # PXI_LBR6 write 
    pxi_init(com_tFPGA, com_PXCT)
    for test in pxi_lbr6_write_test:
        print "PXI_LBR6 test write 0x%X" % (test['write']),
        com_tFPGA.set_pxi_lbr6_direction(test['direction'])
        com_tFPGA.set_pxi_lbr6(test['write'])
        val = com_PXCT.get_pxi_lbl6()
        if val != test['read']:
            ret_error = "Incorrect PXI_LBR6 value read: 0x%X, expected: 0x%X" % (val, test['read'])
            print "(read: 0x%X, expected: 0x%X)" % (val, test['read']),
            print_error()
        else:
            print_ok()
    print ""
    # PXI_LBR6 read
    pxi_init(com_tFPGA, com_PXCT)
    for test in pxi_lbr6_read_test:
        print "PXI_LBR6 test read 0x%X" % (test['read']),
        com_PXCT.set_pxi_lbl6_direction(test['direction'])
        com_PXCT.set_pxi_lbl6(test['write'])
        val = com_tFPGA.get_pxi_lbr6()
        if val != test['read']:
            ret_error = "Incorrect PXI_LBR6 value read: 0x%X, expected: 0x%X" % (val, test['read'])
            print "(read: 0x%X, expected: 0x%X)" % (val, test['read']),
            print_error()
        else:
            print_ok()
    print ""
    pxi_init(com_tFPGA, com_PXCT)
    return ret_error

pxi_ga_read_test = [
    { 'read':0x01}
]

def pxi_ga_test(com_tFPGA, com_PXCT):
    ret_error = None
    # PXI_GA read
    pxi_init(com_tFPGA, com_PXCT)
    for test in pxi_ga_read_test:
        print "PXI_GA test read 0x%X" % (test['read']),
        val = com_tFPGA.get_pxi_ga()
        if val != test['read']:
            ret_error = "Incorrect PXI_GA value read: 0x%X, expected: 0x%X" % (val, test['read'])
            print "(read: 0x%X, expected: 0x%X)" % (val, test['read']),
            print_error()
        else:
            print_ok()
    print ""
    pxi_init(com_tFPGA, com_PXCT)
    return ret_error

pxi_star_read_test = [
    { 'write':0x0, 'read':0x0, 'direction':0x0}, 
    { 'write':0x1, 'read':0x1, 'direction':0x0},
    { 'write':0x0, 'read':0x0, 'direction':0x0}, 
    { 'write':0x1, 'read':0x1, 'direction':0x0}
]

def pxi_star_test(com_tFPGA, com_PXCT):
    ret_error = None
    # PXI_STAR read
    pxi_init(com_tFPGA, com_PXCT)
    com_PXCT.set_pxi_star_select(0) # 0:star_out, 1:pxi_clk10
    com_PXCT.set_pxi_star_enable(1)
    for test in pxi_star_read_test:
        print "PXI_STAR test read 0x%X" % (test['read']),
        com_PXCT.set_pxi_star(test['write'])
        val = com_tFPGA.get_pxi_star()
        if val != test['read']:
            ret_error = "Incorrect PXI_STAR value read: 0x%X, expected: 0x%X" % (val, test['read'])
            print "(read: 0x%X, expected: 0x%X)" % (val, test['read']),
            print_error()
        else:
            print_ok()
    print ""
    com_PXCT.set_pxi_star_enable(0)
    pxi_init(com_tFPGA, com_PXCT)
    return ret_error

pxi_star_10MHz_read_test = [
    0b11111000001111100000111110000011,
    0b01111100000111110000011111000001,
    0b00111110000011111000001111100000,
    0b00011111000001111100000111110000,
    0b00001111100000111110000011111000,
    0b00000111110000011111000001111100,
    0b10000011111000001111100000111110,
    0b11000001111100000111110000011111,
    0b11100000111110000011111000001111,
    0b11110000011111000001111100000111
]

def pxi_star_10MHz_test(com_tFPGA, com_PXCT):
    ret_error = None
    # PXI_STAR 10MHz read
    pxi_init(com_tFPGA, com_PXCT)
    com_PXCT.set_pxi_star_select(1) # 0:star_out, 1:pxi_clk10
    com_PXCT.set_pxi_star_enable(1)
    for test in range(0, 20):
        val = com_tFPGA.get_pxi_star_rec()
        print "PXI_STAR 10MHz test read %s" % (format(val, '#034b')),
        if val in pxi_star_10MHz_read_test:
            print_ok()
        else:
            ret_error = "Incorrect PXI_STAR value read: %s" % (format(val, '#034b'))
            print_error()
    print ""
    com_PXCT.set_pxi_star_enable(0)
    pxi_init(com_tFPGA, com_PXCT)
    return ret_error


def main (default_directory='.'):

    print "Test10 (PXI) Start"
    init_test_time = time.time()
    ret_error = None
    abort = None

    # Front panel COM port connected to the PXCT module in slot 2
    print ""
    com_PXCT_p = PXCT(device = COM0_DEVICE)
    com_PXCT_p.connect()
    response = com_PXCT_p.id()
    print 'PXCT (Peripheral) ID response: %s' % (response),
    exp_response = 'CPE-PXCT-19916'
    if exp_response not in response:
        abort = 'Incorrect PXCT (Peripheral Slot) response.'
        print_error()
    else:
        print_ok()
    slot = com_PXCT_p.get_pxi_ga()
    print 'PXCT in slot: %d' % (slot),
    if slot != 2:
        abort = 'PXCT (Peripheral Slot) located in wrong slot.'
        print_error()
    else:
        print_ok()
    if abort:
        com_PXCT_p.close()
        raise PtsCritical(abort)

    # Internal serial port connected to the onboard FPGA
    print ""
    com_tFPGA = PXIeCOMe_tFPGA()
    com_tFPGA.connect()
    response = com_tFPGA.id()
    print 'FPGA ID response: %s' % (response),
    exp_response = 'PXIeCOMe tFPGA'
    if exp_response not in response:
        abort = 'Incorrect PXIeCOMe tFPGA response.'
        print_error()
    else:
        print_ok()
    if abort:
        com_tFPGA.close()
        raise PtsCritical(abort)

    # Front panel USB4 port connected USB-Serial adapter and to the
    # PXCT module in System Timing slot
    print ""
    com_PXCT_t = PXCT(device = VCOM_DEVICE)
    com_PXCT_t.connect()
    response = com_PXCT_t.id()
    print 'PXCT (System Timing) ID response: %s' % (response),
    exp_response = 'CPE-PXCT-19917'
    if exp_response not in response:
        abort = 'Incorrect PXCT (System Timing Slot) response.'
        print_error()
    else:
        print_ok()
    slot = com_PXCT_t.get_pxi_ga()
    print 'PXCT in slot: %d' % (slot)
    if abort:
        com_PXCT_t.close()
        raise PtsCritical(abort)

    print ""

    # PXI_TRIG test
    ret = pxi_trig_test(com_tFPGA, com_PXCT_p)
    if ret != None:
        ret_error = ret
    
    # PXI_LBR6 test
    ret = pxi_lbr6_test(com_tFPGA, com_PXCT_p)
    if ret != None:
        ret_error = ret

    # PXI_GA test
    ret = pxi_ga_test(com_tFPGA, com_PXCT_p)
    if ret != None:
        ret_error = ret

    # PXI_STAR test        
    ret = pxi_star_test(com_tFPGA, com_PXCT_t)
    if ret != None:
        ret_error = ret

    # PXI_STAR 10MHz test        
    ret = pxi_star_10MHz_test(com_tFPGA, com_PXCT_t)
    if ret != None:
        ret_error = ret

    com_tFPGA.close()
    com_PXCT_p.close()
    com_PXCT_t.close()

    print ""
    end_test_time = time.time()
    print "End of Test10 (PXI)"
    print "RESULT: [{}]".format("FAIL" if ret_error else "OK")
    print 'Test10 elapsed time: {:.2f} seconds'.format(end_test_time-init_test_time)
    if ret_error:
        raise PtsError(ret_error)
    

if __name__ == '__main__' :
    main()
